#coding: utf-8

import licant.util
from licant.core import WrongAction

import sys
from optparse import OptionParser

import os


def cli_argv_parse(argv):
    parser = OptionParser()
    parser.add_option("-d", "--debug", action="store_true",
                      default=False, help="print full system commands")
    parser.add_option("-t", "--trace", action="store_true",
                      default=False, help="print trace information")
    parser.add_option("-j", "--threads", default=1,
                      help="amount of threads for executor")
    parser.add_option("-q", "--quite", action="store_true",
                      default=False, help="don`t print shell operations")

    parser.add_option("--printruntime", action="store_true", default=False)

    opts, args = parser.parse_args(argv)
    return opts, args


def cliexecute(default, colorwrap=False, argv=sys.argv[1:], core=licant.core.core):
    if colorwrap:
        print(licant.util.green("[start]"))

    opts, args = cli_argv_parse(argv)

    core.runtime["debug"] = opts.debug or opts.trace
    core.runtime["trace"] = opts.trace
    core.runtime["quite"] = opts.quite

    cpu_count = os.cpu_count()
    core.runtime["threads"] = cpu_count if opts.threads == 'j' else int(opts.threads)

    if opts.printruntime:
        print("PRINT RUNTIME:", core.runtime)

    if len(args) == 0:
        if default is None:
            licant.util.error("default target isn't set")

        try:
            target = core.get(default)
            ret = target.invoke(target.default_action, critical=True)
        except licant.core.WrongAction as e:
            licant.util.error("Enough default action " + licant.util.yellow(
                target.default_action) + " in default target " + licant.util.yellow(_default))

    if len(args) == 1:
        fnd = args[0]
        if fnd in core.targets:
            try:
                target = core.get(fnd)
                if not hasattr(target, "default_action"):
                    licant.util.error("target {} hasn't default_action (actions: {})"
                        .format(licant.util.yellow(args[0]), licant.util.get_actions(target)))
                ret = target.invoke(target.default_action, critical=True)
            except licant.core.WrongAction as e:
                licant.util.error("target.default_action")
        else:
            try:
                target = core.get(default)
                ret = target.invoke(fnd, critical=True)
            except licant.core.WrongAction as e:
                licant.util.error("Can't find routine " + licant.util.yellow(fnd) +
                                  ". Enough target or default target action with same name.")

    if len(args) >= 2:
        tgt = args[0]
        act = args[1]
        act_args = args[2:] 

        if core.runtime["debug"]:
            print("licant.ex: tgt:{}, act:{}, args:{}".format(tgt, act, act_args))

        try:
            target = licant.core.core.get(tgt)
            ret = target.invoke(act, *act_args, critical=True)
        except licant.core.WrongAction as e:
            licant.util.error("Can't find action " + licant.util.yellow(args[1]) +
                              " in target " + licant.util.yellow(args[0]))

    if colorwrap:
        print(licant.util.yellow("[finish]"))



from licant.core import core
import licant.make
import os


class binutils:
    def __init__(self, cxx, cc, ld, ar, objdump, moc=None):
        self.cc = cc
        self.cxx = cxx
        self.ld = ld
        self.ar = ar
        self.objdump = objdump
        self.moc = moc

    def __repr__(self):
        return str(self.__dict__)


host_binutils = binutils(
    cxx="c++",
    cc="cc",
    ld="ld",
    ar="ar",
    objdump="objdump",
    moc="moc"  # Qt support
)


class options:
    def __init__(self, binutils=host_binutils, include_paths=None, defines=None, cxx_flags="", cc_flags="", ld_flags="", ld_srcs_add="", ldscripts=None):
        self.binutils = binutils
        self.incopt = licant.util.flag_prefix("-I", include_paths)
        self.defopt = licant.util.flag_prefix("-D", defines)
        self.ldscripts = licant.util.flag_prefix("-T", ldscripts)
        self.cxx_flags = cxx_flags
        self.cc_flags = cc_flags
        self.ld_flags = ld_flags
        self.ld_srcs_add = ld_srcs_add
        self.execrule = "{opts.binutils.cxx} {opts.ld_flags} -Wl,--start-group {srcs} {opts.ld_srcs_add} -Wl,--end-group -o {tgt} {opts.ldscripts}"
        self.dynlibrule = "{opts.binutils.cxx} --shared {opts.ld_flags} -Wl,--start-group {srcs} {opts.ld_srcs_add} -Wl,--end-group -o {tgt} {opts.ldscripts}"
        self.cxxobjrule = "{opts.binutils.cxx} -c {src} -o {tgt} {opts.incopt} {opts.defopt} {opts.cxx_flags}"
        self.ccobjrule = "{opts.binutils.cc} -c {src} -o {tgt} {opts.incopt} {opts.defopt} {opts.cc_flags}"
        self.cxxdeprule = "{opts.binutils.cxx} -MM {src} > {tgt} {opts.incopt} {opts.defopt} {opts.cxx_flags}"
        self.ccdeprule = "{opts.binutils.cc} -MM {src} > {tgt} {opts.incopt} {opts.defopt} {opts.cc_flags}"
        self.mocrule = "{opts.binutils.moc} {src} > {tgt}"


cxx_ext_list = ["cpp", "cxx"]
cc_ext_list = ["cc", "c"]
asm_ext_list = ["asm", "s", "S"]


def object(src, tgt, opts=options(), type=None, deps=None, message="OBJECT {tgt}"):
    if deps is None:
        deps = [src]

    if type is None:
        ext = os.path.basename(src).split('.')[-1]

        if ext in cxx_ext_list:
            type = "cxx"
        elif ext in cc_ext_list:
            type = "cc"
        elif ext in asm_ext_list:
            type = "asm"
        else:
            print("Unrecognized extention: {}".format(licant.util.red(ext)))
            exit(-1)
    if type == "cxx":
        build = licant.make.Executor(opts.cxxobjrule)
    elif type == "cc":
        build = licant.make.Executor(opts.ccobjrule)
    elif type == "asm":
        build = licant.make.Executor(opts.ccobjrule)
    else:
        print(licant.util.red("Unrecognized extention"))
        exit(-1)
    core.add(licant.make.FileTarget(
        opts=opts,
        tgt=tgt,
        src=src,
        deps=deps,
        build=build,
        message=message
    ))


def moc(src, tgt, opts=options(), type=None, deps=None, message="MOC {tgt}"):
    if deps is None:
        deps = [src]

    core.add(licant.make.FileTarget(
        opts=opts,
        tgt=tgt,
        src=src,
        deps=deps,
        build=licant.make.Executor(opts.mocrule),
        message=message
    ))


def depend(src, tgt, opts=options(), type=None, deps=None, message="DEPENDS {tgt}", **kwargs):
    if deps is None:
        deps = [src]

    if type is None:
        ext = os.path.basename(src).split('.')[-1]

        if ext in cxx_ext_list:
            type = "cxx"
        elif ext in cc_ext_list:
            type = "cc"
        elif ext in asm_ext_list:
            type = "asm"
        else:
            print("Unrecognized extention: {}".format(licant.util.red(ext)))
            exit(-1)
    if type == "cxx":
        build = licant.make.Executor(opts.cxxdeprule)
    elif type == "cc":
        build = licant.make.Executor(opts.ccdeprule)
    elif type == "asm":
        build = licant.make.Executor(opts.ccdeprule)
    else:
        print(licant.util.red("Unrecognized extention"))
        exit(-1)

    core.add(licant.make.FileTarget(
        opts=opts,
        tgt=tgt,
        src=src,
        deps=deps,
        build=build,
        message=message,
        **kwargs
    ))


def executable(tgt, srcs, opts=options(), message="EXECUTABLE {tgt}"):
    core.add(licant.make.FileTarget(
        opts=opts,
        tgt=tgt,
        build=licant.make.Executor(opts.execrule),
        srcs=" ".join(srcs),
        deps=srcs,
        message=message
    ))


def dynamic_library(tgt, srcs, opts=options(), message="DYNLIB {tgt}"):
    core.add(licant.make.FileTarget(
        opts=opts,
        tgt=tgt,
        build=licant.make.Executor(opts.dynlibrule),
        srcs=" ".join(srcs),
        deps=srcs,
        message=message
    ))


def make_gcc_binutils(pref):
    return binutils(
        cxx=pref + "-g++",
        cc=pref + "-gcc",
        ld=pref + "-ld",
        ar=pref + "-ar",
        objdump=pref + "-objdump"
    )

def disassembler(target, *args):
    if len(args) <= 1:
        print("usage: disasm object_path asmlist_path")

    _target = core.get(args[0])
    _target.makefile()

    cmd = "{} -D {} > {}".format(_target.opts.binutils.objdump, _target.tgt, args[1])
    print(cmd)
    os.system(cmd)


binutils_target = licant.core.Target(
    tgt="binutils",
    deps=[],
    disasm=disassembler,
    actions={"disasm"}
)

core.add(binutils_target)




from licant.cli import cliexecute as ex

from licant.core import Core, Target, UpdatableTarget, UpdateStatus
from licant.core import core as default_core
from licant.core import routine_decorator as routine

from licant.make import copy, fileset
from licant.cxx_modules import application as cxx_application
from licant.cxx_modules import shared_library as cxx_shared_library

from licant.modules import module, submodule
from licant.util import error

import licant.scripter
from licant.libs import include

__version__ = '1.1.1'


def subtree(tgt):
	return default_core.subtree(tgt)


def execute(path):
    licant.scripter.scriptq.execute(path)


def execute_recursive(*argv, **kwargs):
    licant.scripter.scriptq.execute_recursive(*argv, **kwargs)


def about():
    return "I'm Licant"
